@@ -35,6 +35,7 @@
 #include <libxml/HTMLtree.h>
 #include <libxml/debugXML.h>
 #include <libxml/xmlerror.h>
+#include <libxml/globals.h>
 
 #ifdef LIBXML_DEBUG_ENABLED
 static int debug = 0;
@@ -43,8 +44,11 @@ static int copy = 0;
 static int sax = 0;
 static int repeat = 0;
 static int noout = 0;
+#ifdef LIBXML_PUSH_ENABLED
 static int push = 0;
+#endif /* LIBXML_PUSH_ENABLED */
 static char *encoding = NULL;
+static int options = 0;
 
 xmlSAXHandler emptySAXHandlerStruct = {
     NULL, /* internalSubset */
@@ -73,7 +77,12 @@ xmlSAXHandler emptySAXHandlerStruct = {
     NULL, /* xmlParserError */
     NULL, /* getParameterEntity */
     NULL, /* cdataBlock */
-    NULL  /* externalSubset */
+    NULL, /* externalSubset */
+    1,    /* initialized */
+    NULL, /* private */
+    NULL, /* startElementNsSAX2Func */
+    NULL, /* endElementNsSAX2Func */
+    NULL  /* xmlStructuredErrorFunc */
 };
 
 xmlSAXHandlerPtr emptySAXHandler = &emptySAXHandlerStruct;
@@ -373,7 +382,8 @@ startElementDebug(void *ctx ATTRIBUTE_UNUSED, const xmlChar *name, const xmlChar
 		while ((attlen = strlen((char*)att)) > 0) {
 		    outlen = sizeof output - 1;
 		    htmlEncodeEntities(output, &outlen, att, &attlen, '\'');
-		    fprintf(stdout, "%.*s", outlen, output);
+		    output[outlen] = 0;
+		    fprintf(stdout, "%s", (char *) output);
 		    att += attlen;
 		}
 		fprintf(stdout, "'");
@@ -591,6 +601,11 @@ xmlSAXHandler debugSAXHandlerStruct = {
     fatalErrorDebug,
     getParameterEntityDebug,
     cdataDebug,
+    NULL,
+    1,
+    NULL,
+    NULL,
+    NULL,
     NULL
 };
 
@@ -608,6 +623,7 @@ parseSAXFile(char *filename) {
     /*
      * Empty callbacks for checking
      */
+#ifdef LIBXML_PUSH_ENABLED
     if (push) {
 	FILE *f;
 
@@ -622,7 +638,7 @@ parseSAXFile(char *filename) {
 	    res = fread(chars, 1, 4, f);
 	    if (res > 0) {
 		ctxt = htmlCreatePushParserCtxt(emptySAXHandler, NULL,
-			    chars, res, filename, 0);
+			    chars, res, filename, XML_CHAR_ENCODING_NONE);
 		while ((res = fread(chars, 1, size, f)) > 0) {
 		    htmlParseChunk(ctxt, chars, res, 0);
 		}
@@ -648,7 +664,7 @@ parseSAXFile(char *filename) {
 		res = fread(chars, 1, 4, f);
 		if (res > 0) {
 		    ctxt = htmlCreatePushParserCtxt(debugSAXHandler, NULL,
-				chars, res, filename, 0);
+				chars, res, filename, XML_CHAR_ENCODING_NONE);
 		    while ((res = fread(chars, 1, size, f)) > 0) {
 			htmlParseChunk(ctxt, chars, res, 0);
 		    }
@@ -664,6 +680,7 @@ parseSAXFile(char *filename) {
 	    }
 	}
     } else {	
+#endif /* LIBXML_PUSH_ENABLED */
 	doc = htmlSAXParseFile(filename, NULL, emptySAXHandler, NULL);
 	if (doc != NULL) {
 	    fprintf(stdout, "htmlSAXParseFile returned non-NULL\n");
@@ -680,7 +697,9 @@ parseSAXFile(char *filename) {
 		xmlFreeDoc(doc);
 	    }
 	}
+#ifdef LIBXML_PUSH_ENABLED
     }
+#endif /* LIBXML_PUSH_ENABLED */
 }
 
 static void
@@ -704,7 +723,7 @@ parseAndPrintFile(char *filename) {
 	    res = fread(chars, 1, 4, f);
 	    if (res > 0) {
 		ctxt = htmlCreatePushParserCtxt(NULL, NULL,
-			    chars, res, filename, 0);
+			    chars, res, filename, XML_CHAR_ENCODING_NONE);
 		while ((res = fread(chars, 1, size, f)) > 0) {
 		    htmlParseChunk(ctxt, chars, res, 0);
 		}
@@ -715,7 +734,7 @@ parseAndPrintFile(char *filename) {
 	    fclose(f);
 	}
     } else {	
-	doc = htmlParseFile(filename, NULL);
+	doc = htmlReadFile(filename, NULL, options);
     }
     if (doc == NULL) {
         xmlGenericError(xmlGenericErrorContext,
@@ -731,6 +750,7 @@ parseAndPrintFile(char *filename) {
 	xmlFreeDoc(tmp);
     }
 
+#ifdef LIBXML_OUTPUT_ENABLED
     /*
      * print it.
      */
@@ -750,6 +770,7 @@ parseAndPrintFile(char *filename) {
 	    htmlDocDump(stdout, doc);
 #endif
     }	
+#endif /* LIBXML_OUTPUT_ENABLED */
 
     /*
      * free it.
@@ -769,8 +790,10 @@ int main(int argc, char **argv) {
 #endif
 	    if ((!strcmp(argv[i], "-copy")) || (!strcmp(argv[i], "--copy")))
 	    copy++;
+#ifdef LIBXML_PUSH_ENABLED
 	else if ((!strcmp(argv[i], "-push")) || (!strcmp(argv[i], "--push")))
 	    push++;
+#endif /* LIBXML_PUSH_ENABLED */
 	else if ((!strcmp(argv[i], "-sax")) || (!strcmp(argv[i], "--sax")))
 	    sax++;
 	else if ((!strcmp(argv[i], "-noout")) || (!strcmp(argv[i], "--noout")))
@@ -818,7 +841,9 @@ int main(int argc, char **argv) {
 	printf("\t--sax : debug the sequence of SAX callbacks\n");
 	printf("\t--repeat : parse the file 100 times, for timing\n");
 	printf("\t--noout : do not print the result\n");
+#ifdef LIBXML_PUSH_ENABLED
 	printf("\t--push : use the push mode parser\n");
+#endif /* LIBXML_PUSH_ENABLED */
 	printf("\t--encode encoding : output in the given encoding\n");
     }
     xmlCleanupParser();
@@ -828,7 +853,7 @@ int main(int argc, char **argv) {
 }
 #else /* !LIBXML_HTML_ENABLED */
 #include <stdio.h>
-int main(int argc, char **argv) {
+int main(int argc ATTRIBUTE_UNUSED, char **argv ATTRIBUTE_UNUSED) {
     printf("%s : HTML support not compiled in\n", argv[0]);
     return(0);
 }
