@@ -171,6 +171,7 @@ int main(int argc, char **argv) {
     if (valid != 0) xmlDoValidityCheckingDefaultValue = 1;
     xmlLoadExtDtdDefaultValue |= XML_DETECT_IDS;
     xmlLoadExtDtdDefaultValue |= XML_COMPLETE_ATTRS;
+    xmlSubstituteEntitiesDefaultValue = 1;
     if (nocdata != 0) {
 	xmlDefaultSAXHandlerInit();
 	xmlDefaultSAXHandler.cdataBlock = NULL;
@@ -219,7 +220,7 @@ int main(int argc, char **argv) {
 }
 #else
 #include <stdio.h>
-int main(int argc, char **argv) {
+int main(int argc ATTRIBUTE_UNUSED, char **argv ATTRIBUTE_UNUSED) {
     printf("%s : XPath/Debug support not compiled in\n", argv[0]);
     return(0);
 }
