@@ -89,6 +89,9 @@
 #include <libxml/relaxng.h>
 #include <libxml/xmlschemas.h>
 #endif
+#ifdef LIBXML_PATTERN_ENABLED
+#include <libxml/pattern.h>
+#endif
 
 #ifndef XML_XML_DEFAULT_CATALOG
 #define XML_XML_DEFAULT_CATALOG "file:///etc/xml/catalog"
@@ -128,6 +131,7 @@ static int repeat = 0;
 static int insert = 0;
 #ifdef  LIBXML_HTML_ENABLED
 static int html = 0;
+static int xmlout = 0;
 #endif
 static int htmlout = 0;
 #ifdef LIBXML_PUSH_ENABLED
@@ -153,11 +157,16 @@ static int nocatalogs = 0;
 #endif
 #ifdef LIBXML_READER_ENABLED
 static int stream = 0;
+static int walker = 0;
 #endif /* LIBXML_READER_ENABLED */
 static int chkregister = 0;
 #ifdef LIBXML_SAX1_ENABLED
 static int sax1 = 0;
 #endif /* LIBXML_SAX1_ENABLED */
+#ifdef LIBXML_PATTERN_ENABLED
+static const char *pattern = NULL;
+static xmlPatternPtr patternc = NULL;
+#endif
 static int options = 0;
 
 /*
@@ -603,7 +612,7 @@ static void processNode(xmlTextReaderPtr reader) {
 
     name = xmlTextReaderConstName(reader);
     if (name == NULL)
-	name = xmlStrdup(BAD_CAST "--");
+	name = BAD_CAST "--";
 
     value = xmlTextReaderConstValue(reader);
 
@@ -618,6 +627,13 @@ static void processNode(xmlTextReaderPtr reader) {
     else {
 	printf(" %s\n", value);
     }
+#ifdef LIBXML_PATTERN_ENABLED
+    if (patternc) {
+        if (xmlPatternMatch(patternc, xmlTextReaderCurrentNode(reader)) == 1) {
+	    printf("Node matches pattern %s\n", pattern);
+	}
+    }
+#endif
 }
 
 static void streamFile(char *filename) {
@@ -638,12 +654,12 @@ static void streamFile(char *filename) {
 	if (base == (void *) MAP_FAILED)
 	    return;
 
-	input = xmlParserInputBufferCreateStatic((char *) base, info.st_size,
-	                                      XML_CHAR_ENCODING_NONE);
-	reader = xmlNewTextReader(input, filename);
+	reader = xmlReaderForMemory(base, info.st_size, filename,
+	                            NULL, options);
     } else
 #endif
-	reader = xmlNewTextReaderFilename(filename);
+	reader = xmlReaderForFile(filename, NULL, options);
+
 
     if (reader != NULL) {
 #ifdef LIBXML_VALID_ENABLED
@@ -678,7 +694,11 @@ static void streamFile(char *filename) {
 	}
 	ret = xmlTextReaderRead(reader);
 	while (ret == 1) {
-	    if (debug)
+	    if ((debug)
+#ifdef LIBXML_PATTERN_ENABLED
+	        || (patternc)
+#endif
+	       )
 		processNode(reader);
 	    ret = xmlTextReaderRead(reader);
 	}
@@ -734,6 +754,39 @@ static void streamFile(char *filename) {
     }
 #endif
 }
+
+static void walkDoc(xmlDocPtr doc) {
+    xmlTextReaderPtr reader;
+    int ret;
+
+    reader = xmlReaderWalker(doc);
+    if (reader != NULL) {
+	if ((timing) && (!repeat)) {
+	    startTimer();
+	}
+	ret = xmlTextReaderRead(reader);
+	while (ret == 1) {
+	    if ((debug)
+#ifdef LIBXML_PATTERN_ENABLED
+	        || (patternc)
+#endif
+	       )
+		processNode(reader);
+	    ret = xmlTextReaderRead(reader);
+	}
+	if ((timing) && (!repeat)) {
+	    endTimer("walking through the doc");
+	}
+	xmlFreeTextReader(reader);
+	if (ret != 0) {
+	    printf("failed to walk through the doc\n");
+	    progresult = 1;
+	}
+    } else {
+	fprintf(stderr, "Failed to crate a reader from the document\n");
+	progresult = 1;
+    }
+}
 #endif /* LIBXML_READER_ENABLED */
 
 /************************************************************************
@@ -954,7 +1007,7 @@ static void parseAndPrintFile(char *filename, xmlParserCtxtPtr rectxt) {
 	if ((timing) && (!repeat)) {
 	    startTimer();
 	}
-	xmlXIncludeProcess(doc);
+	xmlXIncludeProcessFlags(doc, options);
 	if ((timing) && (!repeat)) {
 	    endTimer("Xinclude processing");
 	}
@@ -1006,8 +1059,15 @@ static void parseAndPrintFile(char *filename, xmlParserCtxtPtr rectxt) {
 	}    
     }else
 #endif /* LIBXML_VALID_ENABLED */
+#ifdef LIBXML_READER_ENABLED
+    if (walker) {
+        walkDoc(doc);
+    }
+#endif /* LIBXML_READER_ENABLED */
 #ifdef LIBXML_OUTPUT_ENABLED
     if (noout == 0) {
+        int ret;
+
 	/*
 	 * print it.
 	 */
@@ -1017,6 +1077,45 @@ static void parseAndPrintFile(char *filename, xmlParserCtxtPtr rectxt) {
 	    if ((timing) && (!repeat)) {
 		startTimer();
 	    }
+#ifdef LIBXML_VALID_ENABLED
+            if ((html) && (!xmlout)) {
+		if (compress) {
+		    htmlSaveFile(output ? output : "-", doc);
+		}
+		else if (encoding != NULL) {
+		    if ( format ) {
+			htmlSaveFileFormat(output ? output : "-", doc, encoding, 1);
+		    }
+		    else {
+			htmlSaveFileFormat(output ? output : "-", doc, encoding, 0);
+		    }
+		}
+		else if (format) {
+		    htmlSaveFileFormat(output ? output : "-", doc, NULL, 1);
+		}
+		else {
+		    FILE *out;
+		    if (output == NULL)
+			out = stdout;
+		    else {
+			out = fopen(output,"wb");
+		    }
+		    if (out != NULL) {
+			if (htmlDocDump(out, doc) < 0)
+			    progresult = 6;
+
+			if (output != NULL)
+			    fclose(out);
+		    } else {
+			fprintf(stderr, "failed to open %s\n", output);
+			progresult = 6;
+		    }
+		}
+		if ((timing) && (!repeat)) {
+		    endTimer("Saving");
+		}
+	    } else
+#endif
 #ifdef HAVE_SYS_MMAN_H
 	    if (memory) {
 		xmlChar *result;
@@ -1047,14 +1146,25 @@ static void parseAndPrintFile(char *filename, xmlParserCtxtPtr rectxt) {
 	    }
 	    else if (encoding != NULL) {
 	        if ( format ) {
-		    xmlSaveFormatFileEnc(output ? output : "-", doc, encoding, 1);
+		    ret = xmlSaveFormatFileEnc(output ? output : "-", doc,
+		                               encoding, 1);
 		}
 		else {
-		    xmlSaveFileEnc(output ? output : "-", doc, encoding);
+		    ret = xmlSaveFileEnc(output ? output : "-", doc, encoding);
+		}
+		if (ret < 0) {
+		    fprintf(stderr, "failed save to %s\n",
+		            output ? output : "-");
+		    progresult = 6;
 		}
 	    }
 	    else if (format) {
-		xmlSaveFormatFile(output ? output : "-", doc, 1);
+		ret = xmlSaveFormatFile(output ? output : "-", doc, 1);
+		if (ret < 0) {
+		    fprintf(stderr, "failed save to %s\n",
+		            output ? output : "-");
+		    progresult = 6;
+		}
 	    }
 	    else {
 		FILE *out;
@@ -1355,6 +1465,7 @@ static void usage(const char *name) {
 #endif /* LIBXML_OUTPUT_ENABLED */
 #ifdef LIBXML_HTML_ENABLED
     printf("\t--html : use the HTML parser\n");
+    printf("\t--xmlout : force to use the XML serializer when using --html\n");
 #endif
 #ifdef LIBXML_PUSH_ENABLED
     printf("\t--push : use the push mode of the parser\n");
@@ -1375,7 +1486,7 @@ static void usage(const char *name) {
 #ifdef LIBXML_CATALOG_ENABLED
     printf("\t--catalogs : use SGML catalogs from $SGML_CATALOG_FILES\n");
     printf("\t             otherwise XML Catalogs starting from \n");
-    printf("\t         " XML_XML_DEFAULT_CATALOG " are activated by default\n");
+    printf("\t         %s are activated by default\n", XML_XML_DEFAULT_CATALOG);
     printf("\t--nocatalogs: deactivate all catalogs\n");
 #endif
     printf("\t--auto : generate a small doc on the fly\n");
@@ -1386,7 +1497,11 @@ static void usage(const char *name) {
     printf("\t--dtdattr : loaddtd + populate the tree with inherited attributes \n");
 #ifdef LIBXML_READER_ENABLED
     printf("\t--stream : use the streaming interface to process very large files\n");
+    printf("\t--walker : create a reader and walk though the resulting doc\n");
 #endif /* LIBXML_READER_ENABLED */
+#ifdef LIBXML_PATTERN_ENABLED
+    printf("\t--pattern pattern_value : test the pattern support\n");
+#endif
     printf("\t--chkregister : verify the node registration code\n");
 #ifdef LIBXML_SCHEMAS_ENABLED
     printf("\t--relaxng schema : do RelaxNG validation against the schema\n");
@@ -1485,6 +1600,10 @@ main(int argc, char **argv) {
 	         (!strcmp(argv[i], "--html"))) {
 	    html++;
         }
+	else if ((!strcmp(argv[i], "-xmlout")) ||
+	         (!strcmp(argv[i], "--xmlout"))) {
+	    xmlout++;
+        }
 #endif /* LIBXML_HTML_ENABLED */
 	else if ((!strcmp(argv[i], "-loaddtd")) ||
 	         (!strcmp(argv[i], "--loaddtd"))) {
@@ -1505,16 +1624,19 @@ main(int argc, char **argv) {
 	         (!strcmp(argv[i], "--postvalid"))) {
 	    postvalid++;
 	    loaddtd++;
+	    options |= XML_PARSE_DTDLOAD;
 	} else if ((!strcmp(argv[i], "-dtdvalid")) ||
 	         (!strcmp(argv[i], "--dtdvalid"))) {
 	    i++;
 	    dtdvalid = argv[i];
 	    loaddtd++;
+	    options |= XML_PARSE_DTDLOAD;
 	} else if ((!strcmp(argv[i], "-dtdvalidfpi")) ||
 	         (!strcmp(argv[i], "--dtdvalidfpi"))) {
 	    i++;
 	    dtdvalidfpi = argv[i];
 	    loaddtd++;
+	    options |= XML_PARSE_DTDLOAD;
         }
 #endif /* LIBXML_VALID_ENABLED */
 	else if ((!strcmp(argv[i], "-dropdtd")) ||
@@ -1553,7 +1675,7 @@ main(int argc, char **argv) {
 	else if ((!strcmp(argv[i], "-xinclude")) ||
 	         (!strcmp(argv[i], "--xinclude"))) {
 	    xinclude++;
-	    /* options |= XML_PARSE_XINCLUDE; */
+	    options |= XML_PARSE_XINCLUDE;
 	}
 #endif
 #ifdef LIBXML_OUTPUT_ENABLED
@@ -1620,6 +1742,11 @@ main(int argc, char **argv) {
 	         (!strcmp(argv[i], "--stream"))) {
 	     stream++;
 	}
+	else if ((!strcmp(argv[i], "-walker")) ||
+	         (!strcmp(argv[i], "--walker"))) {
+	     walker++;
+             noout++;
+	}
 #endif /* LIBXML_READER_ENABLED */
 #ifdef LIBXML_SAX1_ENABLED
 	else if ((!strcmp(argv[i], "-sax1")) ||
@@ -1645,7 +1772,13 @@ main(int argc, char **argv) {
 #endif
         } else if ((!strcmp(argv[i], "-nonet")) ||
                    (!strcmp(argv[i], "--nonet"))) {
-	    xmlSetExternalEntityLoader(xmlNoNetExternalEntityLoader);
+	    options |= XML_PARSE_NONET;
+#ifdef LIBXML_PATTERN_ENABLED
+        } else if ((!strcmp(argv[i], "-pattern")) ||
+                   (!strcmp(argv[i], "--pattern"))) {
+	    i++;
+	    pattern = argv[i];
+#endif
 	} else {
 	    fprintf(stderr, "Unknown option %s\n", argv[i]);
 	    usage(argv[0]);
@@ -1761,7 +1894,18 @@ main(int argc, char **argv) {
 	    endTimer("Compiling the schemas");
 	}
     }
-#endif
+#endif /* LIBXML_SCHEMAS_ENABLED */
+#ifdef LIBXML_PATTERN_ENABLED
+    if (pattern != NULL) {
+        patternc = xmlPatterncompile((const xmlChar *) pattern, NULL, 0, NULL);
+	if (patternc == NULL) {
+	    xmlGenericError(xmlGenericErrorContext,
+		    "Pattern %s failed to compile\n", pattern);
+            progresult = 7;
+	    pattern = NULL;
+	}
+    }
+#endif /* LIBXML_PATTERN_ENABLED */
     for (i = 1; i < argc ; i++) {
 	if ((!strcmp(argv[i], "-encode")) ||
 	         (!strcmp(argv[i], "--encode"))) {
@@ -1795,6 +1939,13 @@ main(int argc, char **argv) {
 	    i++;
 	    continue;
         }
+#ifdef LIBXML_PATTERN_ENABLED
+        if ((!strcmp(argv[i], "-pattern")) ||
+	    (!strcmp(argv[i], "--pattern"))) {
+	    i++;
+	    continue;
+	}
+#endif
 	if ((timing) && (repeat))
 	    startTimer();
 	/* Remember file names.  "-" means stdin.  <sven@zen.org> */
@@ -1845,6 +1996,10 @@ main(int argc, char **argv) {
     if (wxschemas != NULL)
 	xmlSchemaFree(wxschemas);
     xmlRelaxNGCleanupTypes();
+#endif
+#ifdef LIBXML_PATTERN_ENABLED
+    if (patternc != NULL)
+        xmlFreePattern(patternc);
 #endif
     xmlCleanupParser();
     xmlMemoryDump();
